#include <qapplication.h>
#include <qmultilineedit.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpixmap.h>
#include <qstring.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qwhatsthis.h>
#include <qfiledialog.h>
#include <qtextstream.h>
#include <qmessagebox.h>

#include "jsemain.h"
#include "my_fps.h"
#include "my_editor.h"
#include "my_about.h"
#include "sub_convert.h"
#include "sub_shift.h"
#include "sub_showtime.h"
#include "sub_fpschange.h"
#include "sub_correct.h"

// XPM'y (ikonki)
#include "./graph/newtext.xpm"
#include "./graph/opentext.xpm"
#include "./graph/savetext.xpm"
#include "./graph/convertformat.xpm"
#include "./graph/shifttime.xpm"
#include "./graph/showtimechange.xpm"
#include "./graph/changefps.xpm"
#include "./graph/correctsub.xpm"

   /* ------------------------------------- */

QSubEditMain::QSubEditMain(QString filename) : QMainWindow(0, APPWINDOWNAME, WDestructiveClose)
{
  QPixmap icoNew, icoOpen, icoSave, icoConvert, icoShTime, icoShowTCh, icoChFps, icoCorrect;
  icoNew     = QPixmap (newtext_xpm);
  icoOpen    = QPixmap (opentext_xpm);
  icoSave    = QPixmap (savetext_xpm);
  icoConvert = QPixmap (convertformat_xpm);
  icoShTime  = QPixmap (shifttime_xpm);
  icoShowTCh = QPixmap (showtimechange_xpm);
  icoChFps   = QPixmap (changefps_xpm);
  icoCorrect = QPixmap (correctsub_xpm);

// toolbar
  
  toolbar = new QToolBar(this, "ToolBar");
  QToolButton *toolBtnNew  = new QToolButton(icoNew,       "Nowy plik napisw (Ctrl+N)",       QString::null, this, SLOT(newFile()),  toolbar, "nowy plik");
  QToolButton *toolBtnOpen = new QToolButton(icoOpen,      "Otwrz plik napisw (Ctrl+O)",     QString::null, this, SLOT(open()),     toolbar, "otwarcie pliku");
  QToolButton *toolBtnSave = new QToolButton(icoSave,      "Zapisz plik napisw (Ctrl+S)",     QString::null, this, SLOT(save()),     toolbar, "zapisanie pliku");
  toolbar->addSeparator();
  toolbar->addSeparator();
  QToolButton *toolBtnCnvt = new QToolButton(icoConvert,   "Zmie format napisw (Ctrl+T)",                QString::null, this, SLOT(subtitlesConvert()),     toolbar, "zmiana formatu");
  QToolButton *toolBtnShTm = new QToolButton(icoShTime ,   "Przesu czas wywietlenia napisw (Ctrl+W)",   QString::null, this, SLOT(shiftTime()),            toolbar, "przesuniecie");
  QToolButton *toolBtnShift= new QToolButton(icoShowTCh,   "Zmie czas wywietlania napisw (Ctrl+D)",     QString::null, this, SLOT(changeShowTime()),       toolbar, "czas wyswietlania");
  QToolButton *toolBtnChFps= new QToolButton(icoChFps,     "Zmie wspczynnik FPS napisw (Ctrl+Q)",      QString::null, this, SLOT(changeSubFps()),         toolbar, "wspolczynnik FPS");
  QToolButton *toolBtnCorr = new QToolButton(icoCorrect,   "Popraw napisy (Ctrl+R)",                       QString::null, this, SLOT(correctSubtitles()),     toolbar, "popraw napisy");

//  QWhatsThis::whatsThisButton(toolbar);

// main menu
  QPopupMenu *menuFile = new QPopupMenu(this);
  menuBar()->insertItem("&Plik", menuFile);
  
   //menuFile->setWhatsThis(
   menuFile->insertItem(icoNew  , "&Nowy plik napisw",      this, SLOT(newFile()),          CTRL+Key_N);//, "trrrrrrrrrrrrrr");
   menuFile->insertItem(icoOpen , "&Otwrz plik napisw",    this, SLOT(open()),             CTRL+Key_O);
   menuFile->insertSeparator();
   menuFile->insertItem(icoSave , "&Zapisz plik",            this, SLOT(save()),             CTRL+Key_S);
   menuFile->insertItem(          "Zapisz &jako...",         this, SLOT(saveAs()),           SHIFT+CTRL+Key_S);
   menuFile->insertSeparator();
   menuFile->insertItem(          "&Zamknij",                this, SLOT(close()),            CTRL+Key_F4);
   menuFile->insertItem(          "Zamknij &wszystkie",      qApp, SLOT(closeAllWindows()),  ALT+Key_F4);
   
  QPopupMenu *menuSubtiles = new QPopupMenu(this);
  menuBar()->insertItem("&Napisy", menuSubtiles); 
   menuSubtiles->insertItem(                 "&Okrel FPS napisw",                      this, SLOT(specifyFps()),       CTRL+Key_I);
   menuSubtiles->insertSeparator();
   menuSubtiles->insertItem(icoConvert,      "&Konwertuj format napisw",                this, SLOT(subtitlesConvert()), CTRL+Key_T);
   menuSubtiles->insertItem(icoShTime ,      "&Przesu czas wywietlenia napisw",       this, SLOT(shiftTime()),        CTRL+Key_D);
   menuSubtiles->insertItem(icoShowTCh,      "&Zmie czas wywietlania napisw",         this, SLOT(changeShowTime()),   CTRL+Key_W);
   menuSubtiles->insertItem(icoChFps,        "Zmie wspczynnik &FPS napisw",          this, SLOT(changeSubFps()),     CTRL+Key_Q);
   menuSubtiles->insertItem(icoCorrect,      "Popraw &napisy",                           this, SLOT(correctSubtitles()), CTRL+Key_R);
 
  menuBar()->insertSeparator();
  QPopupMenu *menuHelp = new QPopupMenu(this);
  menuBar()->insertItem("&Pomoc", menuHelp); 
   menuHelp->insertItem(          "&Informacje o programie...",               this, SLOT(showAbout()),        Key_F1);

  editor = new MyEditor(this, "editor");
  editor->setFocus();
  setCentralWidget(editor);
  resize(500, 600);
  if (QString(filename)!=DEFAULTFILENAME)
    load(filename);
  else
    setFileName(filename);
  editor->setEdited(false);
}
   /* ------------------------------------- */

QSubEditMain::~QSubEditMain()
{
}

//////////////////////////////////////////// ZDARZENIA //////////////////////////////////////////////
void QSubEditMain::closeEvent(QCloseEvent *e)
{
  if (!editor->edited())
  {
    e->accept();
    return;
  }
  switch
  (
    QMessageBox::information(this, APPNAME, "Napisy zostay zmienione i nie zapisane!", "Zapisz teraz", "Anuluj", "Nie zapisuj", 0, 1)
  )
  {
    case 0:
      save();
      e->accept();
      break;
    case 1:
    default:
      e->ignore();
      break;
    case 2:
      e->accept();
      break;
  }
}
   /* ------------------------------------- */

//////////////////////////////////////////// SLOTY //////////////////////////////////////////////////
void QSubEditMain::setFileName(QString filename)
{
  setCaption(QString(APPNAME)+" ("+filename+")");
}
   /* ------------------------------------- */

void QSubEditMain::newFile() // slot
{
  //qDebug("newFile");
  (new QSubEditMain(DEFAULTFILENAME))->show();
}
   /* ------------------------------------- */

void QSubEditMain::open()
{
  //qDebug("open");
  QString filename = QFileDialog::getOpenFileName(QString::null, "*.txt", this);
  if (!filename.isEmpty())
    load(filename);
}
   /* ------------------------------------- */

void QSubEditMain::load(QString filename) // slot
{
  if (editor->edited())
  switch
  (
    QMessageBox::information(this, APPNAME, "Napisy zostay zmienione i nie zapisane!", "Zapisz teraz", "Anuluj", "Nie zapisuj", 0, 1)
  )
  {
    case 0:
      save();
      break;
    case 1:
    default:
      return;
      break;
    case 2:
      break;
  }

  this->filename = filename;
  //qDebug("open filename");
  QFile filen(filename);
  if (!filen.open(IO_Raw | IO_ReadWrite | IO_Translate)) return;
  editor->clear();
  setFileName(filename);
  QTextStream texts(&filen); 
  while (!texts.eof())
  {
    QString linia = texts.readLine();
    editor->append(linia);
  }
  filen.close();
  editor->setEdited(false);
  mustSpecifyFps();
}
   /* ------------------------------------- */

void QSubEditMain::save() // slot
{
  //qDebug("save");
  if (filename.isEmpty())
  {
    saveAs();
    return;
  }
  QString text = editor->text();
  QFile filen(filename);
  if (!filen.open(IO_WriteOnly))
  {
    QMessageBox::warning(this, APPNAME, "Wystpi problem z zapisem do pliku \""+filename+"\"!", 0);
    return;
  }
  
  QTextStream texts(&filen);
  texts << text;
  filen.close();
  
  editor->setEdited(false);
  setFileName(filename);
}
   /* ------------------------------------- */

void QSubEditMain::saveAs() // slot
{
  //qDebug("save As");
  QString filen = QFileDialog::getSaveFileName(QString::null, "*.txt", this);
  if (!filen.isEmpty())
  {
    filename = filen;
    save();
  }
}
   /* ------------------------------------- */

void QSubEditMain::showAbout()
{
  MyAbout *dialog = new MyAbout(NULL);
  dialog->exec();
  delete dialog;
}
   /* ------------------------------------- */

void QSubEditMain::mustSpecifyFps()
{
  MyFps dialog (NULL, &editor->fps_value, false);
  dialog.exec();
}
   /* ------------------------------------- */

void QSubEditMain::specifyFps()
{
  MyFps *dialog = new MyFps (NULL, &editor->fps_value, true);
  dialog->exec();
  delete dialog;
}
   /* ------------------------------------- */

void QSubEditMain::subtitlesConvert()
{
// 1-00:00:00  2-0:00:00  3-{00}{00}  4-[00][00]
  SubConvert *dialog = new SubConvert (&editor->convertDestFormat);
  if (dialog->exec())
    editor->linesChange(1);
  delete dialog;
}
   /* ------------------------------------- */

void QSubEditMain::shiftTime()
{
  SubShift *dialog = new SubShift (&editor->shiftTimeValue, 
                   &editor->shiftTimeShowTimeChange, &editor->shiftTimeHideTimeChange);
  if (dialog->exec())
    editor->linesChange(2);
  delete dialog;
}
   /* ------------------------------------- */

void QSubEditMain::changeShowTime()
{
  SubShowtime *dialog = new SubShowtime (&editor->showTimeChangeTime, 
                   &editor->showTimeChangeEnterEmpty, &editor->showTimeChangeOnlyEmpty, 
		   &editor->showTimeChangeDependentOnLen, &editor->showTimeChangeBlock);
  if (dialog->exec())
  {
    editor->linesChange(3);
    
  }
  delete dialog;
}
   /* ------------------------------------- */

void QSubEditMain::changeSubFps()
{
  SubFpschange *dialog = new SubFpschange (&editor->fpsChangeSource, &editor->fpsChangeDest);
  if (dialog->exec())
    editor->linesChange(4);
  delete dialog;
}
   /* ------------------------------------- */

void QSubEditMain::correctSubtitles()
{
  

  SubCorrect *dialog = new SubCorrect (
    &editor->correctDelDblSpaces,&editor->correctDelEolSpaces,&editor->correctDelNumSpaces,
    &editor->correctCutColors,&editor->correctBlockShowTime,
    &editor->correctPlacePunctuation,&editor->correctDashArrabgement);
  if (dialog->exec())
    editor->linesChange(5);
  delete dialog;
}
